// Modified
/* 

   CS 202, Fall 2006
   Homework#9, Problem 3

   Author: Harish Kumar (shapes.cpp modified to include square object)

*/

/*
 * Copyright (c) 1993-1999, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 *
 *  shapes.cpp
 *
 *  This program demonstrates interaction with geometric primitives. 
 */
#include <cstdlib>
#include <GL/glut.h>
#include <iostream>
using std::cout;
using std::endl;


class Shape {
  public:
  virtual ~Shape() { };		// virtual destructor
  virtual void draw(void) = 0;
  virtual void move(int x, int y) = 0;
};
class Triangle : public Shape {
public:
  Triangle(int b=20, int h=30, int x=10, int y=10);
  virtual void draw(void);
  virtual void move(int x, int y);
private:
  int Base, Height, X, Y;
};
Triangle::Triangle(int b, int h, int x, int y) 
{
  Base=b; Height=h; X=x; Y=y; 
}
void Triangle::draw(void) 
{
  glBegin(GL_LINE_STRIP);
    glVertex2f((GLfloat) X, (GLfloat) Y);
    glVertex2f((GLfloat) X, (GLfloat) (Y+Height));
    glVertex2f((GLfloat) (X+Base), (GLfloat) Y);
    glVertex2f((GLfloat) X, (GLfloat) Y);
  glEnd();
}
void Triangle::move(int xinc, int yinc) { X += xinc; Y += yinc; }

const int MaxShapes = 10;		// maximum number of objects
static Shape *ShapePtr[MaxShapes];	// array of Shape class pointers
static int NShapes = 0;			// actual number of objects
static int Selected = -1;		// subscript of selected object

// Modifications adding new class called Square
class Square : public Shape {
public:
  Square(int s=30, int x=10, int y=10);
  virtual void draw(void);
  virtual void move(int x, int y);
private:
  int Side, X, Y;
};
Square::Square(int s, int x, int y) 
{
  Side=s; X=x; Y=y; 
}
void Square::draw(void) 
{
  glBegin(GL_LINE_STRIP);
    glVertex2f((GLfloat) X, (GLfloat) Y);
    glVertex2f((GLfloat) X, (GLfloat) (Y+Side));
    glVertex2f((GLfloat) (X+Side), (GLfloat) (Y+Side));
    glVertex2f((GLfloat) (X+Side), (GLfloat) Y);
    glVertex2f((GLfloat) X, (GLfloat) Y);
  glEnd();
}
void Square::move(int xinc, int yinc) { X += xinc; Y += yinc; }

void init(void) 
{
   glClearColor (0.0, 0.0, 0.0, 0.0);
   glShadeModel (GL_FLAT);
}

void display(void)
{  // callback for drawing display
   int i;
   glClear (GL_COLOR_BUFFER_BIT);

/* draw the shapes in the ShapePtr array */
   for(i=0;i<NShapes;i++) {
     if (i == Selected)	// select color is white
       glColor3f (1.0, 1.0, 1.0);
     else		// default color is green
       glColor3f (0.0, 1.0, 0.0);
     ShapePtr[i]->draw();
   }
   glFlush ();
}

void reshape (int w, int h)
{  // callback for resizing window

   glViewport (0, 0, (GLsizei) w, (GLsizei) h);
   glMatrixMode (GL_PROJECTION);
   glLoadIdentity ();
   gluOrtho2D (0.0, (GLdouble) w, 0.0, (GLdouble) h);
}

void keyboard(unsigned char key, int x, int y)
{  // callback for keyboard events
   switch (key) {
      case 27:    // escape
                exit(0); // end program
      case '0':   // delete selected object from array
                if (Selected > -1) {
                   delete ShapePtr[Selected]; NShapes--;
                  // close up empty space in array
                   for (int i=Selected;i<NShapes;i++) 
                      ShapePtr[i]=ShapePtr[i+1];
                   ShapePtr[NShapes]=0;
                  // reset selected object
                   if (Selected == NShapes) 
                      Selected = NShapes - 1;
                } break;
      case '1':   // add new triangle
                if (NShapes < MaxShapes) {
                   Selected = NShapes; NShapes++;
                   ShapePtr[Selected] = new Triangle;
                } break;
	  case '2':   // add new square
		         if (NShapes < MaxShapes) {                           // 
Modifications
					 Selected = NShapes; NShapes++;
					 ShapePtr[Selected] = new Square;
				 } break;
      case 'l':   // move left
      case 'L': if (Selected > -1) ShapePtr[Selected]->move(-1,0); break;
      case 'r':   // move right
      case 'R': if (Selected > -1) ShapePtr[Selected]->move(1,0); break;
      case 'U':   // move up
      case 'u': if (Selected > -1) ShapePtr[Selected]->move(0,1); break;
      case 'd':   // move down
      case 'D': if (Selected > -1) ShapePtr[Selected]->move(0,-1); break;
      case ' ':   // increment selection
                if (NShapes > 0) Selected = (Selected+1) % NShapes;
                break;
   }
   glutPostRedisplay();		// refresh the display
}

int main(int argc, char** argv)
{
   cout << "Shapes program menu:\n";
   cout << "  0: delete selected object\n";
   cout << "  1: add new triangle\n";
   cout << "  2: add new square\n"; // modification adding square option
   cout << "  r: move selected object right\n";
   cout << "  l: move selected object left\n";
   cout << "  u: move selected object up\n";
   cout << "  d: move selected object down\n";
   cout << "  space: select next object in sequence\n";
   cout << "  escape: exit program\n\n";
   cout << "Cursor must be in graphics window to recognize menu commands" 
        << endl;
   glutInit(&argc, argv);
   glutInitDisplayMode (GLUT_SINGLE | GLUT_RGB);
   glutInitWindowSize (400, 150); 
   glutInitWindowPosition (100, 100);
   glutCreateWindow (argv[0]);
   init ();
   glutDisplayFunc(display); 
   glutReshapeFunc(reshape);
   glutKeyboardFunc(keyboard);
   //ShapePtr[0] = new Triangle;
   //NShapes++;
   glutMainLoop();
   return 0;
}


